use std::sync::{Arc, Mutex, RwLock};

use dashmap::{DashMap, DashSet};

use super::modules::Module;

pub type ItemPath = Vec<String>;

#[derive(Clone)]
pub struct Registry(Arc<RegistryInner>);

impl Registry {
    pub fn new() -> Self {
        Self(Arc::new(RegistryInner::new()))
    }
}

struct RegistryInner {
    /// ids will refer to paths of modules
    paths: RwLock<Vec<ItemPath>>,
    modules: DashMap<ItemPath, Arc<Module>>,
    names: DashMap<String, DashSet<RegistryId>>,
}

impl RegistryInner {
    fn new() -> Self {
        Self {
            paths: RwLock::new(Vec::new()),
            modules: DashMap::new(),
            names: DashMap::new(),
        }
    }
}

pub enum RegistryId {
    /// refer to the modlue item itself, the name must be the same as the last item of the path
    Module(usize),
    /// refer to a def in a module
    /// name must refer to name of child of module
    Def(usize),
}
