use std::{path::PathBuf, thread};

use crossbeam::channel::{Receiver, Sender};

use super::{
    error::Error,
    files::{FileId, Files, Loc},
    nodes,
    registry::Registry,
};

pub(super) struct WorldCreationPool {
    workers: Vec<Worker>,
    tx: Sender<Job>,
}

impl WorldCreationPool {
    pub fn new(files: Files, reg: Registry) -> Self {
        let mut workers = Vec::new();
        let (tx, rx) = crossbeam::channel::unbounded();

        for i in 0..num_cpus::get() {
            workers.push(Worker::new(
                i,
                files.clone(),
                reg.clone(),
                JobSender(tx.clone()),
                rx.clone(),
            ))
        }

        Self { workers, tx }
    }
}

pub struct JobSender(Sender<Job>);

enum Job {
    ParseFileMod {
        file: FileId,
        decl: Option<Loc<nodes::Mod>>,
    },
    OpenFile(PathBuf),
}

struct Worker {
    id: usize,
    // thead: thread::JoinHandle<Vec<Error>>,
    thread: thread::JoinHandle<()>,
}

pub struct WorkerCtx {
    errors: Vec<Error>,
    pub files: Files,
    pub reg: Registry,
    pub tx: JobSender,
}

impl Worker {
    fn new(id: usize, files: Files, reg: Registry, sender: JobSender, rx: Receiver<Job>) -> Self {
        let ctx = WorkerCtx {
            errors: Vec::new(),
            files,
            reg,
            tx: sender,
        };

        let thread_handle = thread::spawn(move || {
            for msg in rx {
                match msg {
                    Job::ParseFileMod { file, decl } => todo!(),
                    Job::OpenFile(path) => todo!(),
                }
            }
        });

        Self {
            id,
            thread: thread_handle,
        }
    }
}
